#include "demo.h"
#include "ui_demo.h"
#include "colorbutton.h"

#include <QApplication>

#include <QGraphicsRectItem>
#include <QGraphicsEllipseItem>

#include <QFileDialog>
#include <QClipboard>
#include <QXmlStreamReader>
#include <QXmlStreamWriter>

#include <colorbutton.h>
#include <toolbutton.h>
#include <view.h>
#include <io.h>
#include <property.h>
#include <tree.h>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent) ,
    ui (new Ui::MainWindow)
{
    ui->setupUi(this);

    ui->hsplitter->setStretchFactor(0, 1);
    ui->hsplitter->setStretchFactor(1, 4);
    ui->hsplitter->setStretchFactor(2, 1);

    ui->vsplitter->setStretchFactor(0, 3);
    ui->vsplitter->setStretchFactor(1, 1);

    ui->palette->removeTab (0);

    QToolBar * toolPage = new QToolBar;
    addToolButtons (toolPage);
    ui->palette->addTab (toolPage, "Tools");
    ui->palette->setCurrentWidget(toolPage);

    QToolBar * colorPage = new QToolBar;
    addColorButtons (colorPage);
    ui->palette->addTab (colorPage, "Colors");
    // ui->palette->setCurrentWidget (colorPage);

    ui->statusbar->showMessage (QString ("Qt ") + qVersion());

    // scene = new QGradientScene;
    scene = new Scene;
    ui->graphicsView->setScene (scene);
    scene->setSceneRect(0, 0, 800, 600);
    ui->graphicsView->setAcceptDrops (true);
    // scene->addLine(0, 0, 100, 200, QColor ("red"));

    openFile (":/data/abc.xml");
}

MainWindow::~MainWindow()
{
    delete ui;
}

/* open /save */

void MainWindow::on_actionOpen_triggered ()
{
    QString fileName = QFileDialog::getOpenFileName (this, "Open file");
    if (fileName != "")
        openFile (fileName);
}

void MainWindow::openFile (QString fileName)
{
    QFile f (fileName);
    if (f.open (QFile::ReadOnly))
    {
        QXmlStreamReader r (&f);
        readXml (r, scene);
    }
    refreshTree ();
}

void MainWindow::on_actionSave_triggered ()
{
    QString fileName = QFileDialog::getSaveFileName (this, "Save file");
    if (fileName != "")
        saveFile (fileName);
}

void MainWindow::saveFile (QString fileName)
{
    QFile f (fileName);
    if (f.open (QFile::WriteOnly))
    {
        QXmlStreamWriter w (&f);
        writeXml (w, scene);
    }
}

/* clipboard */

void MainWindow::on_actionCopy_triggered ()
{
    QString code = "";
    QXmlStreamWriter writer (& code);
    writeBegin (writer);

    for (QGraphicsItem * item : scene->selectedItems ())
    {
        writeItem (writer, item);
    }

    writeEnd (writer);

    QMimeData * mimeData = new QMimeData;
    mimeData->setData (shapeFormat, code.toLatin1 ());

    QClipboard * clip = QApplication::clipboard ();
    clip->setMimeData (mimeData);
}

void MainWindow::on_actionPaste_triggered()
{
    QClipboard * clip = QApplication::clipboard ();
    const QMimeData * data = clip->mimeData ();
    if (data->hasFormat (shapeFormat))
    {
       QString code = data->data (shapeFormat);
       QXmlStreamReader reader (code);
       readXml (reader, scene);
    }
}

/* tree */

void MainWindow::refreshTree ()
{
    ui->tree->clear();
    QTreeWidgetItem * top = new QTreeWidgetItem ();
    top->setText(0, "data");

    for (QGraphicsItem * item : scene->items (Qt::AscendingOrder))
    {
        TreeNode * node = new TreeNode ();
        node->setText (0, item->toolTip());
        node->item = item;
        top->addChild (node);
    }

    ui->tree->addTopLevelItem (top);
    ui->tree->expandAll();
}

void MainWindow::refreshTable (QGraphicsItem * item)
{
    ui->prop->displayProperties (item);
}

void MainWindow::on_tree_itemDoubleClicked (QTreeWidgetItem * node, int column)
{
    ui->prop->setRowCount(0);
    TreeNode * tree_node = dynamic_cast <TreeNode *> (node);
    if (tree_node != nullptr && tree_node->item != nullptr)
       refreshTable (tree_node->item);
}

void MainWindow::on_actionRun_triggered ()
{
   QGraphicsRectItem * block = new QGraphicsRectItem;
   block->setRect (0, 0, 200, 160);
   block->setPos (100, 100);
   block->setPen (QColor ("orange"));
   block->setBrush (QColor ("yellow"));
   block->setToolTip ("block");
   block->setFlags (QGraphicsItem::ItemIsMovable | QGraphicsEllipseItem::ItemIsSelectable);
   scene->addItem (block);

   for (int i = 1; i <= 2; i++)
   {
       QGraphicsEllipseItem * item = new QGraphicsEllipseItem;
       item->setRect (0, 0, 40, 40);
       item->setPos (40 + (i-1) * 80, 40);
       item->setPen (QColor ("blue"));
       item->setBrush (QColor ("cornflowerblue"));
       item->setToolTip ("item" + QString::number (i));
       item->setFlags (QGraphicsItem::ItemIsMovable  | QGraphicsEllipseItem::ItemIsSelectable);
       item->setParentItem (block);
   }

   refreshTree ();
}

void MainWindow::on_actionQuit_triggered()
{
   close ();
}

int main(int argc, char *argv[])
{
    QApplication a(argc, argv);
    MainWindow w;
    w.show();
    return a.exec();
}


